#include <string.h>
#include "common.h"

#define ACUA_MAGIC "RADACUA000"

int
    main(int argc, char **argv)
{

typedef struct
{
    // UNIX user id
    uid_t         uid                             __attribute__((packed));
    // flags (see FLG_* and *(f) macros)
    word          flags                           __attribute__((packed));
    // area code
    word          phNoArea                        __attribute__((packed));
    // local phone number
    word          phNoLocal                       __attribute__((packed));
    // number of hosts user is logged into
    byte          nHosts                          __attribute__((packed));
    // hosts user is logged into
    word          host[MAX_HOSTS]                 __attribute__((packed));
    // number of logins on each host
    byte          nLogins[MAX_HOSTS]              __attribute__((packed));
    // last login on each host
    time_t        lastLogin[MAX_HOSTS]            __attribute__((packed));
    // last time online for each host
    time_t        lastOnline[MAX_HOSTS]           __attribute__((packed));
    // max logins permitted
    word          maxLogins                       __attribute__((packed));
    // maximum #/minutes to deduct per minute (e.g. even if a user
    // has logins on 5 hosts, you can only deduct 2 minutes if you want...)
    word          maxDeduct                       __attribute__((packed));
    // idle limit in minutes
    int           idleLimit                       __attribute__((packed));
    // PPP user must xfer at least PPPidleBytes in last PPPidleMinutes
    // or be considered idle
    int           PPPidleMinutes                  __attribute__((packed));
    word          PPPidleBytes                    __attribute__((packed));
    // overall time left
    int           tLeft                           __attribute__((packed));
    // overall time limit
    int           tLimit                          __attribute__((packed));
    // how much credit to extend the user (how far beyond their time
    // limit they can go)
    int           credit                          __attribute__((packed));
    // session time left
    int           sLeft                           __attribute__((packed));
    // session time limit
    int           sLimit                          __attribute__((packed));
    // class time left for each time class
    int           cLeft[MAX_TIME_CLASSES]         __attribute__((packed));
    // class time limit for each time class
    int           cLimit[MAX_TIME_CLASSES]        __attribute__((packed));
    // overall bytes transmitted (e.g. uploaded)
    dword         bTx                             __attribute__((packed));
    // overall bytes received (e.g. downloaded)
    dword         bRx                             __attribute__((packed));
    // bytes transmitted limit
    dword         bTxLimit                        __attribute__((packed));
    // bytes received limit
    dword         bRxLimit                        __attribute__((packed));
    // bytes transferred (xmit + recv) limit
    dword         bLimit                          __attribute__((packed));
    // session bytes transmitted
    dword         bStx                            __attribute__((packed));
    // session bytes received
    dword         bSrx                            __attribute__((packed));
    // session bytes transmitted limit
    dword         bStxLimit                       __attribute__((packed));
    // session bytes received limit
    dword         bSrxLimit                       __attribute__((packed));
    // session bytes transferred (xmit + recv) limit
    dword         bSlimit                         __attribute__((packed));
    // bytes xferred in each of last 60 minutes
    // (used by acua_updated to determine idleness)
    word          nBytes[60]                      __attribute__((packed));
    // when user was locked
    time_t        lockDate                        __attribute__((packed));
    // when user was subscribed
    time_t        subscrDate                      __attribute__((packed));
    // When a user is subscribed, data is copied into these fields so that
    // it can be retrieved when the user is unsubscribed.  This allows you
    // to make modifications to a user that will be undone when they are
    // unsubscribed.  e.g. subFlags is a copy of flags, subMaxLogins is
    // a copy of maxLogins, etc...  see acua_subscribe, acua_unSubscribe
    time_t        subExpire                       __attribute__((packed));
    word          subFlags                        __attribute__((packed));
    word          subMaxLogins                    __attribute__((packed));
    word          subMaxDeduct                    __attribute__((packed));
    int           subIdleLimit                    __attribute__((packed));
    int           subPPPidleMinutes               __attribute__((packed));
    word          subPPPidleBytes                 __attribute__((packed));
    int           subTlimit                       __attribute__((packed));
    int           subCredit                       __attribute__((packed));
    int           subSlimit                       __attribute__((packed));
    int           subClimit[MAX_TIME_CLASSES]     __attribute__((packed));
    dword         subBtxLimit                     __attribute__((packed));
    dword         subBrxLimit                     __attribute__((packed));
    dword         subBlimit                       __attribute__((packed));
    dword         subBStxLimit                    __attribute__((packed));
    dword         subBSrxLimit                    __attribute__((packed));
    dword         subBSlimit                      __attribute__((packed));
    // total minutes online
    word          tMinutes                        __attribute__((packed));
    // when account was created
    time_t        creation                        __attribute__((packed));
    // expiry date
    time_t        expire                          __attribute__((packed));
    // second user flags field
    word          flagz                           __attribute__((packed));
    word          subFlagz                        __attribute__((packed));
} oldUserRec;

    FILE         *inFile, *outFile;
    UserRec       ur;
    int           res=0, rec=0;
    char          magic[ACUA_USERS_MAGIC_LEN];

    if (argc < 3)
	errQuit("radacua001-radacua000 <source> <dest>");

    inFile = fopen(argv[1], "rb");
    if (!inFile)
	perrQuit("fopen(%s)", argv[1]);
    outFile = fopen(argv[2], "wb");
    if (!outFile)
	perrQuit("fopen(%s)", argv[2]);

    readConfig();

    // read the magic
    fread(magic, 1, 11, inFile);
    if (strcmp(magic, "RADACUA001")) {
	errQuit("Magic # check failed for input file");
    }

    // write the magic
    fwrite(ACUA_MAGIC, 1, 11, outFile);

    // convert the user records
    for (;;) {
	res = fread(&ur, sizeof(UserRec), 1, inFile);
	rec++;
	if (res != 1)
	    if (feof(inFile)) break; else errQuit("corrupt database file (rec: %d)", rec);
	if (!fwrite(&ur, sizeof(oldUserRec), 1, outFile))
	    perrQuit("fwrite");
    }

    if (fclose(inFile))
	perrQuit("fclose(%s)", argv[1]);
    if (fclose(outFile))
	perrQuit("fclose(%s)", argv[2]);
    return(0);
}
